package com.factory.init;

import com.factory.base.entity.task.FirstCalendar;
import com.factory.base.entity.task.FirstTask;
import com.factory.common.CommonUtil;
import com.factory.service.FirstTaskService;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.FutureTask;

/**
 * 首件任务定时器
 */
@DisallowConcurrentExecution
public class FirstTaskJob implements Job {
    private static Logger log = LoggerFactory.getLogger(FirstTaskJob.class);

    @Override
    public void execute(JobExecutionContext context) {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) context.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
            e.printStackTrace();
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
        List<FutureTask<Integer>> tasks = new ArrayList<>();
        tasks.add(getFirstFutureTask(paramMap));
        tasks.add(get200FutureTask(paramMap));
        try {
            for (FutureTask<Integer> t : tasks) {
                t.get();
            }//等等所有线程结束
        } catch (Throwable e) {
            e.printStackTrace();
        }

    }

    /**
     * day
     *
     * @param paramMap
     * @return
     */
    private FutureTask<Integer> getFirstFutureTask(ConcurrentMap<String, Object> paramMap) {
        FutureTask<Integer> task = new FutureTask<>(new addTask(paramMap, 0));
        Thread thread = new Thread(task);
        thread.start();  //开启线程
        return task;
    }

    /**
     * 200
     *
     * @param paramMap
     * @return
     */
    private FutureTask<Integer> get200FutureTask(ConcurrentMap<String, Object> paramMap) {
        FutureTask<Integer> task = new FutureTask<>(new addTask(paramMap, 1));
        Thread thread = new Thread(task);
        thread.start();  //开启线程
        return task;
    }


    /**
     * 生产任务
     */
    public class addTask implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private Integer typeFlag;


        public addTask(ConcurrentMap<String, Object> paramMap, Integer typeFlag) {
            this.paramMap = paramMap;
            this.typeFlag = typeFlag;
        }

        @Override
        public Integer call() {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            String td = CommonUtil.getDateStr(new Date(), "yyyy-MM-dd");
            List<Long> cals = firstTaskService.getCalenderFirstForType(typeFlag);
            for (Long calId : cals) {
                try {
                    FirstTask task = new FirstTask();
                    task.setTask_time(td);
                    task.setFirst_article_calendar_id(calId);
                    firstTaskService.insertFirstTask(task);
                } catch (Throwable e) {
                    e.getStackTrace();
                    log.error(e.getStackTrace().toString());
                }

            }
            return 0;
        }
    }

}
