package com.factory.service.impl;

import com.factory.base.entity.task.*;
import com.factory.mapper.FirstTaskMapper;
import com.factory.service.FirstTaskService;
import com.factory.service.MsgService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class FirstTaskServiceImp implements FirstTaskService {

    @Autowired
    FirstTaskMapper mapper;
    @Autowired
    MsgService msgService;

    /**
     * 获取对应的规则
     *
     * @param typeFlag
     * @return
     */
    @Override
    public List<Long> getCalenderFirstForType(Integer typeFlag) {
        return mapper.getCalenderFirstForType(typeFlag);
    }

    /**
     * 添加当天任务
     *
     * @param task
     * @return
     */
    @Override
    public int insertFirstTask(FirstTask task) {
        return mapper.insertFirstTask(task);
    }

    /**
     * 需要提醒的工作
     *
     * @param param
     * @return
     */
    @Override
    public List<FirstCalendar> getRemindWork(Map<String, Object> param) {
        return mapper.getRemindWork(param);
    }


    /**
     * 更新提醒状态，不太好的写法但是符合现在的需求
     *
     * @param param
     * @return
     */
    @Override
    public int updateFirstRemindFlag(Map<String, Object> param) {
        return mapper.updateFirstRemindFlag(param);
    }

    /**
     * 添加不工作提醒信息
     *
     * @param map
     * @return
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int addRemindNowork(Map map) {
        int result = mapper.addRemindNowork(map);
        mapper.updateFirstRemindFlag(map);
        return result;
    }

    /**
     * 获取需要提醒人
     *
     * @param map
     * @return
     */
    @Override
    public Long getRemindNoworkToManager(Map map) {
        return mapper.getRemindNoworkToManager(map);
    }


    /**
     * 获取最大审批级别
     *
     * @param map
     * @return
     */
    @Override
    public int getMaxApproveLevel(Map map) {
        return mapper.getMaxApproveLevel(map);
    }

    /**
     * 或许超时没处理记录
     *
     * @param map
     * @return
     */
    @Override
    public List<ApproveEntry> getApproveTaskNoDeal(Map map) {
        return mapper.getApproveTaskNoDeal(map);
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int addTaskForNext(ApproveEntry vo) {
        HashMap map = new HashMap();
        map.put("approvalLevel", vo.getApproval_level() + 1);
        vo.setStatus(2);
        ApprovalUserRes nextUser = mapper.getApprovalUser(map);
        if (nextUser != null) {
            int result = mapper.updateApproveTask(vo);
            if (result > 0) {
                ApproveEntry approve = new ApproveEntry();
                approve.setTask_id(vo.getTask_id());
                approve.setApproval_level(nextUser.getApproval_level());
                approve.setApproval_user_id(nextUser.getUser_id().longValue());
                int status = mapper.addApproveTask(approve);
                if (status > 0) {
                    Date date = new Date();
                    SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
                    //发送微信消息
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(nextUser.getUser_id());
                    sendMsgReq.setTitle("计划放行提醒");
                    sendMsgReq.setContent("您有待放行的计划审批，请登录小程序查看【计划放行】");
                    sendMsgReq.setEndTime(ft.format(date));
                    msgService.sendWechatMessage(sendMsgReq);
                }
                return status;
            }
        }
        return 0;
    }


}
